using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;
using System.Web.Mvc;
using EPiServer.Core;
using EPiServer.Data.Dynamic;
using EPiServer.Samples.XFormSearch.Models;
using EPiServer.Shell.Gadgets;
using EPiServer.Shell.Web;
using EPiServer.XForms;
using System.Globalization;
using System.Xml;
using System.IO;

namespace EPiServer.Samples.XFormSearch
{
    [Gadget(ClientScriptInitMethod = "xformSearch.init", ResourceType = typeof(XFormSearchController))]
    [ScriptResource("Content/Scripts/XFormSearch.js")]
    public class XFormSearchController : Controller
    {
        [Import]
        public DynamicDataStoreFactory StoreFactory { get; set; }

        public static string Title
        {
            get { return LanguageHelper.Translate("/xformsearchgadget/title"); }
        }

        public static string ConfigurationMenuTitle
        {
            get { return LanguageHelper.Translate("/xformsearchgadget/configuration/title"); }
        }

        #region Actions

        public ViewResult Index(Guid gadgetId)
        {
            return View(ViewModel.Create(StoreFactory, Settings.Load(StoreFactory, gadgetId), null));
        }

        public PartialViewResult ShowSearchForm(Guid gadgetId, Guid formId)
        {
            return PartialView("SearchForm", ViewModel.Create(StoreFactory, Settings.Load(StoreFactory, gadgetId), formId));
        }

        public PartialViewResult SearchFormData()
        {
            // Build a dynamic Linq query to run against the XForm Posts store
            Guid gadgetId = new Guid(Request["gadgetId"]);
            Guid formId = new Guid(Request["formId"]);
            string sortBy = Request["sortBy"];
            string sortOrder = Request["sortOrder"];

            SearchResultOption resultOption = (SearchResultOption)Enum.Parse(typeof(SearchResultOption), Request["resultOption"]);
            int page = 1;

            if (!string.IsNullOrEmpty(Request["currentPage"]))
            {
                page = int.Parse(Request["currentPage"]);
            }

            // Get the store for the XForms posts and construct a query

            DynamicDataStore postsStore = XFormData.GetStore(formId);
            var query = from post in postsStore.ItemsAsPropertyBag() select post;
            Dictionary<string, SearchCriterion> searchCriteria = new Dictionary<string, SearchCriterion>();

            foreach (string key in Request.QueryString)
            {
                switch (key)
                {
                    case "formId":
                    case "gadgetId":
                    case "resultOption":
                    case "currentPage":
                    case "pageCount":
                    case "goToPage":
                    case "sortBy":
                    case "sortOrder":
                        continue;

                    default:

                        // We need to copy the key variable to a new variable
                        // otherwise the value it has when the Linq query is executed below
                        // will be used instead of its current value
                        string controlName = key;

                        // Get the input value from the request
                        string value = Request[controlName];

                        // Exract the name of the XForm field the control is for
                        // and whether it is the search requirement or the input
                        string fieldName = null;
                        int index = 0;
                        ControlNameQualifier qualifier = SearchInputHelper.DecodeControlName(controlName, out fieldName, out index);
                        string criterionKey = fieldName + index.ToString();

                        SearchCriterion criterion = null;
                        if (!searchCriteria.TryGetValue(criterionKey, out criterion))
                        {
                            criterion = new SearchCriterion();
                            criterion.FieldName = fieldName;
                            searchCriteria[criterionKey] = criterion;
                        }

                        switch (qualifier)
                        {
                            case ControlNameQualifier.SearchRequirement:
                                criterion.Requirement = SearchInputHelper.GetSearchRequirement(value);
                                break;

                            case ControlNameQualifier.Input:
                                criterion.Input = value;
                                break;

                            case ControlNameQualifier.Input2:
                                criterion.Input2 = value;
                                break;
                        }

                        break;
                }
            }

            // Now build the query with the search criteria
            foreach (SearchCriterion criterion in searchCriteria.Values)
            {
                // We need to copy the SearchCriterion variable to a new variable
                // otherwise the value it has when the Linq query is executed below
                // will be used instead of its current value
                SearchCriterion c = criterion;

                switch (c.Requirement)
                {
                    case SearchRequirement.Equals:
                        query = query.Where(post => (string)post[c.FieldName] == c.Input);
                        break;

                    case SearchRequirement.NotEquals:
                        query = query.Where(post => (string)post[c.FieldName] != c.Input);
                        break;

                    case SearchRequirement.Contains:
                        query = query.Where(post => ((string)post[c.FieldName]).Contains(c.Input));
                        break;

                    case SearchRequirement.NotContains:
                        query = query.Where(post => !((string)post[c.FieldName]).Contains(c.Input));
                        break;

                    case SearchRequirement.StartsWith:
                        query = query.Where(post => ((string)post[c.FieldName]).StartsWith(c.Input));
                        break;

                    case SearchRequirement.NotStartsWith:
                        query = query.Where(post => !((string)post[c.FieldName]).StartsWith(c.Input));
                        break;

                    case SearchRequirement.EndsWith:
                        query = query.Where(post => ((string)post[c.FieldName]).EndsWith(c.Input));
                        break;

                    case SearchRequirement.NotEndsWith:
                        query = query.Where(post => !((string)post[c.FieldName]).EndsWith(c.Input));
                        break;

                    case SearchRequirement.DateEquals:
                        {
                            DateTime d = DateTime.Parse(c.Input);
                            // The input is just a date with the time set to midnight (00.00)
                            // so the compare has the check the record's date is in the range 00.00 to 23.59.999
                            query = query.Where(post => (DateTime)post[c.FieldName] >= d && (DateTime)post[c.FieldName] <= d.AddDays(1).AddMilliseconds(-1));
                        }
                        break;

                    case SearchRequirement.DateNotEquals:
                        {
                            DateTime d = DateTime.Parse(c.Input);
                            // The input is just a date with the time set to midnight (00.00)
                            // so the compare has the check the record's date is not in the range 00.00 to 23.59.999
                            query = query.Where(post => (DateTime)post[c.FieldName] < d || (DateTime)post[c.FieldName] >= d.AddDays(1));
                        }
                        break;

                    case SearchRequirement.DateLessThan:
                        query = query.Where(post => (DateTime)post[c.FieldName] < DateTime.Parse(c.Input));
                        break;

                    case SearchRequirement.DateGreaterThan:
                        {
                            DateTime d = DateTime.Parse(c.Input);
                            // The input is just a date with the time set to midnight (00.00)
                            // so the compare has the check the record's date is greater than 23.59.999
                            query = query.Where(post => (DateTime)post[c.FieldName] >= d.AddDays(1));
                        }
                        break;

                    case SearchRequirement.DateBetween:
                        {
                            DateTime start = DateTime.Parse(c.Input);
                            DateTime end = DateTime.Parse(c.Input2);
                            // The inputs are just a date with the time set to midnight (00.00)
                            // so the compare has the check the record's date is in the range start date 00.00 to end date 23.59.999
                            query = query.Where(post => (DateTime)post[c.FieldName] >= start && (DateTime)post[c.FieldName] <= end.AddDays(1).AddMilliseconds(-1));
                        }
                        break;

                    case SearchRequirement.DateNotBetween:
                        {
                            DateTime start = DateTime.Parse(c.Input);
                            DateTime end = DateTime.Parse(c.Input2);
                            // The inputs are just a date with the time set to midnight (00.00)
                            // so the compare has the check the record's date is not in the range start date 00.00 to end date 23.59.999
                            query = query.Where(post => (DateTime)post[c.FieldName] < start || (DateTime)post[c.FieldName] >= end.AddDays(1));
                        }
                        break;
                }
            }

            Settings settings = Settings.Load(StoreFactory, gadgetId);
            ViewModel model = ViewModel.Create(StoreFactory, settings, formId);

            switch (resultOption)
            {
                case SearchResultOption.ShowResults:
                    query = ApplySortPreferences(sortBy, sortOrder, query);

                    // Get the results in a paged manner
                    int skipCount = (page - 1) * model.ResultsPerPage;
                    int takeCount = model.ResultsPerPage;
                    model.Results = query.Skip(skipCount).Take(takeCount).ToList();
                    model.CurrentState = SearchState.CountAndResults;
                    model.ResultsPage = page;
                    model.ResultCount = query.Count();
                    break;

                case SearchResultOption.ShowCountOnly:
                    model.CurrentState = SearchState.CountOnly;
                    model.ResultCount = query.Count();
                    break;

                case SearchResultOption.CreateCsv:
                    query = ApplySortPreferences(sortBy, sortOrder, query);
                    CreateCsvOutput(query, model, settings);
                    break;

                case SearchResultOption.CreateXml:
                    query = ApplySortPreferences(sortBy, sortOrder, query);
                    CreateXmlOutput(query, model, settings);
                    break;
            }

            return PartialView("SearchResults", model);
        }

        [GadgetAction(ResourceType = typeof(XFormSearchController), TextResourceKey = "ConfigurationMenuTitle")]
        public ViewResult Configuration(Guid gadgetId)
        {
            return View(Settings.Load(StoreFactory, gadgetId));
        }

        public ViewResult SaveConfiguration(Guid gadgetId, int resultsPerPage, string csvDelimiter)
        {
            Settings originalSettings = Settings.Load(StoreFactory, gadgetId);
            originalSettings.ResultsPerPage = resultsPerPage;
            originalSettings.CsvDelimiter = csvDelimiter;
            originalSettings.Save(StoreFactory);
            return View("Index", ViewModel.Create(StoreFactory, originalSettings, null));
        }

        public JavaScriptResult GetLocalizedStrings()
        {
            StringBuilder jsonObject = new StringBuilder();
            jsonObject.Append(String.Format("loadingFormDefinition:\"{0}\",", LanguageHelper.Translate("/xformsearchgadget/clientstatus/loadingformdefinition")));
            jsonObject.Append(String.Format("searchingFormPosts:\"{0}\",", LanguageHelper.Translate("/xformsearchgadget/clientstatus/searchingformposts")));
            return new JavaScriptResult { Script = string.Format("epi.shell.resource.add('EPiServer.Samples.XFormSearch.Strings', {{{0}}});", jsonObject.ToString().Trim(',')) };
        }

        #endregion

        private static IQueryable<PropertyBag> ApplySortPreferences(string sortBy, string sortOrder, IQueryable<PropertyBag> query)
        {
            if (sortOrder == "D")
            {
                query = query.OrderByDescending(post => post[sortBy]);
            }
            else
            {
                query = query.OrderBy(post => post[sortBy]);
            }

            return query;
        }

        private void CreateCsvOutput(IQueryable<PropertyBag> query, ViewModel model, Settings settings)
        {
            Response.Clear();
            Response.AddHeader("content-disposition", string.Format("attachment; filename=XFormSearchResults{0}.csv", DateTime.Now.ToString("s")));
            Response.ContentType = "text/csv";

            StringBuilder output = new StringBuilder();

            // Write the header row
            foreach (var field in model.SelectedFormFields)
            {
                output.AppendFormat("{0}\"{1}\"", settings.CsvDelimiter ?? ",", field.Name);
            }

            // strip leading tab
            output.Remove(0, 1);
            output.Append("\r\n");

            // and write
            Response.Write(output.ToString());

            // read the results in pages of 500 to avoid memory allocation problems
            output.Length = 0;
            int taken = 0;
            IEnumerable<PropertyBag> results = null;

            do
            {
                results = query.Skip(taken).Take(500).ToList();

                foreach (var result in results)
                {
                    foreach (var field in model.SelectedFormFields)
                    {
                        output.AppendFormat("{0}\"{1}\"", settings.CsvDelimiter ?? ",", result[field.Name]);
                    }

                    // strip leading tab
                    output.Remove(0, 1);
                    output.Append("\r\n");

                    // and write
                    Response.Write(output.ToString());

                    // setup for next read
                    output.Length = 0;
                }

                taken += results.Count();
            }
            while (results.Count() == 500);

            Response.End();
        }

        private void CreateXmlOutput(IQueryable<PropertyBag> query, ViewModel model, Settings settings)
        {
            Response.Clear();
            Response.AddHeader("content-disposition", string.Format("attachment; filename=XFormSearchResults{0}.xml", DateTime.Now.ToString("s")));
            Response.ContentType = "text/xml";

            using (XmlWriter writer = XmlWriter.Create(Response.Output))
            {
                writer.WriteStartDocument();
                writer.WriteStartElement("results");

                // read the results in pages of 500 to avoid memory allocation problems
                int taken = 0;
                IEnumerable<PropertyBag> results = null;

                do
                {
                    results = query.Skip(taken).Take(500).ToList();

                    foreach (var result in results)
                    {
                        writer.WriteStartElement("result");

                        foreach (var field in model.SelectedFormFields)
                        {
                            writer.WriteStartElement(field.Name);
                            writer.WriteValue(result[field.Name]);
                            writer.WriteEndElement();
                        }

                        writer.WriteEndElement();
                    }

                    taken += results.Count();
                }
                while (results.Count() == 500);

                writer.WriteEndElement();
                writer.WriteEndDocument();
            }

            Response.End();
        }
    }

    class SearchCriterion
    {
        public SearchRequirement Requirement;
        public string FieldName;
        public string Input;
        public string Input2;
    }
}
