﻿using System;
using System.Collections.Generic;
using System.Web.Mvc;
using EPiServer.Core;

namespace EPiServer.Samples.XFormSearch.Models
{
    public static class SearchInputHelper
    {
        private const string Delimeter = "_#_";        

        public static string GenerateControlName(string baseName, ControlNameQualifier qualifier)
        {
            return baseName + Delimeter + qualifier.ToString();
        }

        public static string GenerateControlName(string baseName, ControlNameQualifier qualifier, int index)
        {
            return GenerateControlName(baseName, qualifier) + Delimeter + index.ToString();
        }

        public static ControlNameQualifier DecodeControlName(string controlName, out string baseName, out int index)
        {
            string[] parts = controlName.Split(new string[] { Delimeter }, StringSplitOptions.None);

            if (parts.Length < 2)
            {
                throw new InvalidOperationException("controlName is not a valid qualified name");
            }

            baseName = parts[0];

            if (parts.Length > 2)
            {
                index = int.Parse(parts[2]);
            }
            else
            {
                index = 0;
            }

            return (ControlNameQualifier)Enum.Parse(typeof(ControlNameQualifier), parts[1]);
        }

        public static IEnumerable<SelectListItem> GenerateSeachRequirementListItems(InputType inputType)
        {
            List<SelectListItem> items = new List<SelectListItem>()
            {
                new SelectListItem()
                {
                    Text = LanguageHelper.Translate("/xformsearchgadget/searchInput/requirement/none"),
                    Value = ""
                }
            };

            switch (inputType)
            {
                case InputType.FreeText:
                    items.Add(new SelectListItem()
                    {
                        Text = LanguageHelper.Translate("/xformsearchgadget/searchInput/requirement/equals"),
                        Value = "="
                    });
                    items.Add(new SelectListItem()
                    {
                        Text = LanguageHelper.Translate("/xformsearchgadget/searchInput/requirement/notequals"),
                        Value = "!="
                    });
                    items.Add(new SelectListItem()
                    {
                        Text = LanguageHelper.Translate("/xformsearchgadget/searchInput/requirement/contains"),
                        Value = "C"
                    });
                    items.Add(new SelectListItem()
                    {
                        Text = LanguageHelper.Translate("/xformsearchgadget/searchInput/requirement/notcontains"),
                        Value = "!C"
                    });
                    items.Add(new SelectListItem()
                    {
                        Text = LanguageHelper.Translate("/xformsearchgadget/searchInput/requirement/startswith"),
                        Value = "S"
                    });
                    items.Add(new SelectListItem()
                    {
                        Text = LanguageHelper.Translate("/xformsearchgadget/searchInput/requirement/notstartswith"),
                        Value = "!S"
                    });
                    items.Add(new SelectListItem()
                    {
                        Text = LanguageHelper.Translate("/xformsearchgadget/searchInput/requirement/endswith"),
                        Value = "E"
                    });
                    items.Add(new SelectListItem()
                    {
                        Text = LanguageHelper.Translate("/xformsearchgadget/searchInput/requirement/notendswith"),
                        Value = "!E"
                    });
                    break;

                case InputType.SingleSelection:
                    items.Add(new SelectListItem()
                    {
                        Text = LanguageHelper.Translate("/xformsearchgadget/searchInput/requirement/selected"),
                        Value = "="
                    });
                    items.Add(new SelectListItem()
                    {
                        Text = LanguageHelper.Translate("/xformsearchgadget/searchInput/requirement/notselected"),
                        Value = "!="
                    });
                    break;

                case InputType.MultiSelection:
                    items.Add(new SelectListItem()
                    {
                        Text = LanguageHelper.Translate("/xformsearchgadget/searchInput/requirement/selected"),
                        Value = "C"
                    });
                    items.Add(new SelectListItem()
                    {
                        Text = LanguageHelper.Translate("/xformsearchgadget/searchInput/requirement/notselected"),
                        Value = "!C"
                    });
                    break;

                case InputType.DateRange:
                    items.Add(new SelectListItem()
                    {
                        Text = LanguageHelper.Translate("/xformsearchgadget/searchInput/requirement/before"),
                        Value = "D<"
                    });
                    items.Add(new SelectListItem()
                    {
                        Text = LanguageHelper.Translate("/xformsearchgadget/searchInput/requirement/after"),
                        Value = "D>"
                    });
                    items.Add(new SelectListItem()
                    {
                        Text = LanguageHelper.Translate("/xformsearchgadget/searchInput/requirement/on"),
                        Value = "D="
                    });
                    items.Add(new SelectListItem()
                    {
                        Text = LanguageHelper.Translate("/xformsearchgadget/searchInput/requirement/noton"),
                        Value = "D!="
                    });
                    items.Add(new SelectListItem()
                    {
                        Text = LanguageHelper.Translate("/xformsearchgadget/searchInput/requirement/between"),
                        Value = "D><"
                    });
                    items.Add(new SelectListItem()
                    {
                        Text = LanguageHelper.Translate("/xformsearchgadget/searchInput/requirement/notbetween"),
                        Value = "D<>"
                    });
                    break;
            }

            return items;
        }

        public static SearchRequirement GetSearchRequirement(string value)
        {
            switch (value)
            {
                case "=":
                    return SearchRequirement.Equals;

                case "!=":
                    return SearchRequirement.NotEquals;

                case "C":
                    return SearchRequirement.Contains;

                case "!C":
                    return SearchRequirement.NotContains;

                case "S":
                    return SearchRequirement.StartsWith;

                case "!S":
                    return SearchRequirement.NotStartsWith;

                case "E":
                    return SearchRequirement.EndsWith;

                case "!E":
                    return SearchRequirement.NotEndsWith;

                case "D=":
                    return SearchRequirement.DateEquals;

                case "D!=":
                    return SearchRequirement.DateNotEquals;

                case "D<":
                    return SearchRequirement.DateLessThan;

                case "D>":
                    return SearchRequirement.DateGreaterThan;

                case "D><":
                    return SearchRequirement.DateBetween;

                case "D<>":
                    return SearchRequirement.DateNotBetween;

                default:
                    return SearchRequirement.None;
            }
        }
    }

    public enum InputType
    {
        FreeText,
        SingleSelection,
        MultiSelection,
        DateRange
    }

    public enum ControlNameQualifier
    {
        Input, Input2, SearchRequirement, Display
    }

    public enum SearchRequirement
    {
        None,
        Equals,
        NotEquals,
        Contains,
        NotContains,
        StartsWith,
        NotStartsWith,
        EndsWith,
        NotEndsWith,
        DateEquals,
        DateNotEquals,
        DateGreaterThan,
        DateLessThan,
        DateBetween,
        DateNotBetween
    }
    
    public class InputControlDescriptor
    {
        public string Name;
        public string Label;
        public InputType InputType;
        public List<KeyValuePair<string, string>> OptionPairs;
    }

    public enum SearchResultOption
    {
        ShowResults,
        ShowCountOnly,
        CreateCsv,
        CreateXml
    }
}
